"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.VisualRegressionTracker = void 0;
const types_1 = require("./types");
const testRunResult_1 = __importDefault(require("./testRunResult"));
const axios_1 = __importDefault(require("axios"));
const helpers_1 = require("./helpers");
class VisualRegressionTracker {
    constructor(explicitConfig) {
        this.config = {
            apiUrl: "",
            apiKey: "",
            project: "",
            branchName: "",
            ciBuildId: "",
            enableSoftAssert: false,
        };
        if (explicitConfig) {
            this.config = explicitConfig;
        }
        else {
            this.config = helpers_1.readConfigFromFile(this.config);
            this.config = helpers_1.readConfigFromEnv(this.config);
        }
        helpers_1.validateConfig(this.config);
        this.axiosConfig = {
            headers: {
                apiKey: this.config.apiKey,
            },
        };
    }
    isStarted() {
        return !!this.buildId && !!this.projectId;
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = {
                branchName: this.config.branchName,
                project: this.config.project,
                ciBuildId: this.config.ciBuildId,
            };
            const build = yield axios_1.default
                .post(`${this.config.apiUrl}/builds`, data, this.axiosConfig)
                .then(this.handleResponse)
                .catch(this.handleException);
            this.buildId = build.id;
            this.projectId = build.projectId;
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isStarted()) {
                throw new Error("Visual Regression Tracker has not been started");
            }
            yield axios_1.default
                .patch(`${this.config.apiUrl}/builds/${this.buildId}`, {}, this.axiosConfig)
                .then(this.handleResponse)
                .catch(this.handleException);
        });
    }
    submitTestResult(test) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isStarted()) {
                throw new Error("Visual Regression Tracker has not been started");
            }
            const data = Object.assign({ buildId: this.buildId, projectId: this.projectId, branchName: this.config.branchName }, test);
            return axios_1.default
                .post(`${this.config.apiUrl}/test-runs`, data, this.axiosConfig)
                .then(this.handleResponse)
                .catch(this.handleException);
        });
    }
    handleResponse(response) {
        return __awaiter(this, void 0, void 0, function* () {
            return response.data;
        });
    }
    handleException(error) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!error.response) {
                throw new Error("No response from server");
            }
            const status = error.response.status;
            switch (status) {
                case 401:
                    throw new Error("Unauthorized");
                case 403:
                    throw new Error("Api key not authenticated");
                case 404:
                    throw new Error("Project not found");
                default:
                    throw new Error(JSON.stringify(error.response.data, null, "\t"));
            }
        });
    }
    track(test) {
        return __awaiter(this, void 0, void 0, function* () {
            const testRunResponse = yield this.submitTestResult(test);
            this.processTestRun(testRunResponse);
            return new testRunResult_1.default(testRunResponse, this.config.apiUrl);
        });
    }
    processTestRun(testRunResponse) {
        let errorMessage;
        switch (testRunResponse.status) {
            case types_1.TestStatus.new: {
                errorMessage = `No baseline: ${testRunResponse.url}`;
                break;
            }
            case types_1.TestStatus.unresolved: {
                errorMessage = `Difference found: ${testRunResponse.url}`;
                break;
            }
        }
        if (errorMessage) {
            if (this.config.enableSoftAssert) {
                // eslint-disable-next-line no-console
                console.error(errorMessage);
            }
            else {
                throw new Error(errorMessage);
            }
        }
    }
}
exports.VisualRegressionTracker = VisualRegressionTracker;
